@@ -23,39 +23,41 @@ __docformat__ = "restructuredtext en"
 import os
 from os.path import join
 
-distname = 'logilab-common'
-modname = 'common'
-subpackage_of = 'logilab'
+distname = "logilab-common"
+modname = "common"
+subpackage_of = "logilab"
 subpackage_master = True
 
-numversion = (1, 6, 4)
-version = '.'.join([str(num) for num in numversion])
+numversion = (1, 7, 0)
+version = ".".join([str(num) for num in numversion])
 
-license = 'LGPL'  # 2.1 or later
-description = ("collection of low-level Python packages and modules"
-               " used by Logilab projects")
+license = "LGPL"  # 2.1 or later
+description = "collection of low-level Python packages and modules" " used by Logilab projects"
 web = "http://www.logilab.org/project/%s" % distname
 mailinglist = "mailto://python-projects@lists.logilab.org"
 author = "Logilab"
 author_email = "contact@logilab.fr"
 
 
-scripts = [join('bin', 'logilab-pytest')]
-include_dirs = [join('test', 'data')]
+scripts = [join("bin", "logilab-pytest")]
+include_dirs = [join("test", "data")]
 
 install_requires = [
-    'setuptools',
+    "setuptools",
+    "mypy-extensions",
+    "typing_extensions",
 ]
 tests_require = [
-    'pytz',
-    'egenix-mx-base',
+    "pytz",
+    "egenix-mx-base",
 ]
 
-if os.name == 'nt':
-    install_requires.append('colorama')
+if os.name == "nt":
+    install_requires.append("colorama")
 
-classifiers = ["Topic :: Utilities",
-               "Programming Language :: Python",
-               "Programming Language :: Python :: 3",
-               "Programming Language :: Python :: 3 :: Only",
-               ]
+classifiers = [
+    "Topic :: Utilities",
+    "Programming Language :: Python",
+    "Programming Language :: Python :: 3",
+    "Programming Language :: Python :: 3 :: Only",
+]
